/******************************************************************************
 * FIRESTARTER - A Processor Stress Test Utility
 * Copyright (C) 2015 TU Dresden, Center for Information Services and High
 * Performance Computing
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: daniel.hackenberg@tu-dresden.de
 *****************************************************************************/


#ifndef ____FIRESTARTER__GLOBAL__H
#define ____FIRESTARTER__GLOBAL__H

/* current version */
#define VERSION_MAJOR 1
#define VERSION_MINOR 3
#define COPYRIGHT_YEAR 2015

#if (defined(linux) || defined(__linux__)) && defined (AFFINITY)
#define _GNU_SOURCE
#include <sched.h>
#endif

#ifdef ENABLE_VTRACING
#include <vt_user.h>
#endif
#ifdef ENABLE_SCOREP
#include <SCOREP_User.h>
#endif

#include <signal.h>
#include <getopt.h>
#include <unistd.h>
#include <pthread.h>
#include <errno.h>
#include <stdio.h>
#include "cpu.h"


#define THREAD_WAIT        1
#define THREAD_WORK        2
#define THREAD_INIT        3
#define THREAD_STOP        4

#define THREAD_INIT_FAILURE 0xffffffff

#define FUNC_NOT_DEFINED   0

/*
 * watchdog timer
 */
typedef struct watchdog_args {
    unsigned long long *high;
    pid_t pid;
    useconds_t period;
    useconds_t load;
    unsigned int timeout;
} watchdog_arg_t;
watchdog_arg_t watchdog_arg;

/** The data structure that holds all the global data.
 */
typedef struct mydata
{                                   
   struct threaddata *threaddata;           
   cpu_info_t *cpuinfo;
   int *thread_comm; 
   volatile unsigned int ack;   
   unsigned int num_threads;
} mydata_t;

/* data needed by each thread */
typedef struct threaddata
{
   volatile mydata_t *data;                                 
   char* bufferMem;                 
   unsigned long long addrMem;      
   unsigned long long addrHigh;             
   unsigned long long buffersizeMem;     
   unsigned int alignment;      
   unsigned int cpu_id;
   unsigned int thread_id;
   unsigned int package;
   unsigned int period;                     
   unsigned char FUNCTION;
} threaddata_t;


#endif
